"""Module with base class for use cases."""

from abc import ABC

from kili.adapters.kili_api_gateway.kili_api_gateway import KiliAPIGateway


class BaseUseCases(ABC):
    """Base Use Cases."""

    def __init__(self, kili_api_gateway: KiliAPIGateway) -> None:
        """Initialize Base Use Cases."""
        self._kili_api_gateway = kili_api_gateway



"""Label gateway common."""
from typing import Dict, List

from kili.adapters.kili_api_gateway.helpers.queries import fragment_builder
from kili.core.graphql.graphql_client import GraphQLClient
from kili.domain.label import LabelId
from kili.domain.types import ListOrTuple
from kili.exceptions import GraphQLError

from .operations import get_annotations_query


def list_annotations(
    graphql_client: GraphQLClient,
    label_id: LabelId,
    *,
    annotation_fields: ListOrTuple[str],
    classification_annotation_fields: ListOrTuple[str] = (),
    ranking_annotation_fields: ListOrTuple[str] = (),
    transcription_annotation_fields: ListOrTuple[str] = (),
    video_annotation_fields: ListOrTuple[str] = (),
    video_classification_fields: ListOrTuple[str] = (),
    video_object_detection_fields: ListOrTuple[str] = (),
    video_transcription_fields: ListOrTuple[str] = (),
) -> List[Dict]:
    """List annotations for a label."""
    query = get_annotations_query(
        annotation_fragment=fragment_builder(annotation_fields),
        classification_annotation_fragment=fragment_builder(classification_annotation_fields),
        ranking_annotation_fragment=fragment_builder(ranking_annotation_fields),
        transcription_annotation_fragment=fragment_builder(transcription_annotation_fields),
        video_annotation_fragment=fragment_builder(video_annotation_fields),
        video_classification_annotation_fragment=fragment_builder(video_classification_fields),
        video_object_detection_annotation_fragment=fragment_builder(video_object_detection_fields),
        video_transcription_annotation_fragment=fragment_builder(video_transcription_fields),
    )
    variables = {"where": {"labelId": label_id}}

    try:
        result = graphql_client.execute(query, variables)
    except GraphQLError as err:
        if "Cannot query field" in str(err):  # not available on LTS
            return []
        raise

    return result["data"]



"""Mixin extending Kili API Gateway class with Asset related operations."""

from typing import Dict, Generator, List

from kili.adapters.kili_api_gateway.asset.formatters import (
    load_asset_json_fields,
)
from kili.adapters.kili_api_gateway.asset.mappers import asset_where_mapper
from kili.adapters.kili_api_gateway.asset.operations import (
    GQL_COUNT_ASSETS,
    GQL_CREATE_UPLOAD_BUCKET_SIGNED_URLS,
    get_assets_query,
)
from kili.adapters.kili_api_gateway.base import BaseOperationMixin
from kili.adapters.kili_api_gateway.helpers.queries import (
    PaginatedGraphQLQuery,
    QueryOptions,
    fragment_builder,
)
from kili.adapters.kili_api_gateway.label.annotation_to_json_response import (
    AnnotationsToJsonResponseConverter,
)
from kili.domain.asset import AssetFilters
from kili.domain.types import ListOrTuple


class AssetOperationMixin(BaseOperationMixin):
    """Mixin extending Kili API Gateway class with Assets related operations."""

    def list_assets(
        self,
        filters: AssetFilters,
        fields: ListOrTuple[str],
        options: QueryOptions,
    ) -> Generator[Dict, None, None]:
        """List assets with given options."""
        added_label_id_field = added_latest_label_id_field = False
        if "labels.jsonResponse" in fields and "labels.id" not in fields:
            fields = (*fields, "labels.id")
            added_label_id_field = True
        if "latestLabel.jsonResponse" in fields and "latestLabel.id" not in fields:
            fields = (*fields, "latestLabel.id")
            added_latest_label_id_field = True

        fragment = fragment_builder(fields)
        query = get_assets_query(fragment)
        where = asset_where_mapper(filters)
        assets_gen = PaginatedGraphQLQuery(self.graphql_client).execute_query_from_paginated_call(
            query, where, options, "Retrieving assets", GQL_COUNT_ASSETS
        )
        assets_gen = (
            load_asset_json_fields(asset, fields, self.http_client) for asset in assets_gen
        )

        if any("jsonResponse" in field for field in fields):
            converter = AnnotationsToJsonResponseConverter(
                graphql_client=self.graphql_client, project_id=filters.project_id
            )
            for asset in assets_gen:
                if "latestLabel.jsonResponse" in fields and asset.get("latestLabel"):
                    converter.patch_label_json_response(
                        asset["latestLabel"], asset["latestLabel"]["id"]
                    )
                    if added_latest_label_id_field:
                        asset["latestLabel"].pop("id")
                if "labels.jsonResponse" in fields:
                    for label in asset.get("labels", []):
                        converter.patch_label_json_response(label, label["id"])
                        if added_label_id_field:
                            label.pop("id")

                yield asset

        else:
            yield from assets_gen

    def count_assets(self, filters: AssetFilters) -> int:
        """Send a GraphQL request calling countIssues resolver."""
        where = asset_where_mapper(filters)
        payload = {"where": where}
        count_result = self.graphql_client.execute(GQL_COUNT_ASSETS, payload)
        count: int = count_result["data"]
        return count

    def create_upload_bucket_signed_urls(self, file_paths: List[str]) -> List[str]:
        """Send a GraphQL request calling createUploadBucketSignedUrls resolver."""
        payload = {
            "filePaths": file_paths,
        }
        urls_response = self.graphql_client.execute(GQL_CREATE_UPLOAD_BUCKET_SIGNED_URLS, payload)
        return urls_response["urls"]
